<?php

include "entities.php";

// This file contains functions for using LuxSci API



/**
 * SendEmail
 */
/*item ($code, $result) = sendEmailOrText($request, $p, $t);

$request:
    attachments: Optional.  If included, this is an array of JSON objects, one for each uploaded file. 
                    The individual JSON objects will have the following keywords and values:
                    - name – The exact filename of the uploaded file
                    - hash – The SHA256 hex-encoded hash of the content of this file.
 
    from_address:    Optional. Email address to use as the "From address" for this message.
    bcc:             Optional. Array of email addresses for the recipients in the "Bcc" of the message.
    cc:              Optional. Array of email addresses for the recipients in the "Cc" of the message.
    subject:         Required.  Subject of this message.  This can be a maximum of 1000 characters long.
    no_tls_only:     If this is passed and is true (1), then no email messages to recipients will be allowed to be sent using "TLS Only"
    body:            If this is passed and is true (1), then no email messages to recipients will be allowed to be sent using "TLS Only"
    to:              Optional. Array of email addresses for the recipients in the "To" of the message.
    from_name:       Optional. The plain text name of the sender.
    receipt:         If set to "1", a read receipt will be requested. For SecureLine Escrow and SecureText,
    body_type:      Optional. Email address to use as the "Reply-To address" for this message.
    reply_address:  Optional. Email address to use as the "Reply-To address" for this message.
    
$p:
   token:  Required. API Token
   secret: Required. API Secret
   host:   Required. API Hostname
   debug:  0|1, If true, emit debugging telemetry
   user:   Optional username
   pass:   Optional password

$t: Auth token of the user.

Returns the HTTP response code and a perl hash reference containing
the response data.

*/

// function definiton to send the message
 	function sendEmailOrText(SendEmailOrTextRequest $request, $p, $t)
    {
      
		$res = API_Request( array_merge($p,
			array('auth' => $t,
				'method' => "POST",
				'path' => '/perl/api/v2/user/' . $p['user'] . '/email/compose/secureline/send',
				'data' => $request->toArray()
			)));
		
        return $res;
    }

	
/**
 * SendEmailWithAttachment
 */
/*item ($code, $result) = sendEmailOrText($request, $p, $t);

$request:
    attachments: Optional.  If included, this is an array of JSON objects, one for each uploaded file. 
                    The individual JSON objects will have the following keywords and values:
                    - name – The exact filename of the uploaded file
                    - hash – The SHA256 hex-encoded hash of the content of this file.
 
    from_address:    Optional. Email address to use as the "From address" for this message.
    bcc:             Optional. Array of email addresses for the recipients in the "Bcc" of the message.
    cc:              Optional. Array of email addresses for the recipients in the "Cc" of the message.
    subject:         Required.  Subject of this message.  This can be a maximum of 1000 characters long.
    no_tls_only:     If this is passed and is true (1), then no email messages to recipients will be allowed to be sent using "TLS Only"
    body:            If this is passed and is true (1), then no email messages to recipients will be allowed to be sent using "TLS Only"
    to:              Optional. Array of email addresses for the recipients in the "To" of the message.
    from_name:       Optional. The plain text name of the sender.
    receipt:         If set to "1", a read receipt will be requested. For SecureLine Escrow and SecureText,
    body_type:      Optional. Email address to use as the "Reply-To address" for this message.
    reply_address:  Optional. Email address to use as the "Reply-To address" for this message.
    
$p:
   token:  Required. API Token
   secret: Required. API Secret
   host:   Required. API Hostname
   debug:  0|1, If true, emit debugging telemetry
   user:   Optional username
   pass:   Optional password

$t: Auth token of the user.
$filepath: attachmentfilepath

Returns the HTTP response code and a perl hash reference containing
the response data.

*/

// function definiton to send the message with attachment
    
	function sendEmailWithAttachment(SendEmailOrTextRequest $request, $p, $t, $filepath)
    {
		
        // get the attachment file path, size and the handle for upload
         
        $handle = fopen($filepath, "rb"); 
        $fsize = filesize($filepath); 
        $contents = fread($handle, $fsize); 
        
        $data = file_get_contents($filepath); // get the file contects of the attachment
        $filename = basename($filepath); // get the actual file name for passing with the request

        //set attachment values
        $attachment = new Attachment();
        $attachment->name = basename($filepath); 
        $attachment->hash =  hash('sha256', $data);;
        $request->attachments =array($attachment);  // assign the attachment to request 

        $method="POST";
        $path ="/perl/api/v2/user/" . $p['user'] . "/email/compose/secureline/send";
        $auth = $t;
        
        //convert to array contents
        $Arraycontents = json_encode($request);



     
        $eol = "\r\n"; //default line-break for mime type
        $BOUNDARY = md5(time()); //random boundaryid, is a separator for each param on my post curl function
        $BODY=""; //init my curl body
        $BODY.= '--'.$BOUNDARY. $eol; //start param header
        $BODY.= 'Content-Type: application/json' . $eol; //set the json part of the request
        // set the luxsci standards for sending the JSON. last Content with 2 $eol, in this case is only 1 content.
        $BODY .= 'Content-Disposition: form-data; name="json"; filename="json.js"' . $eol . $eol; 
        $BODY .= $Arraycontents . $eol;//param data in this case is a simple post data and 1 $eol for the end of the data
        $BODY.= '--'.$BOUNDARY. $eol; // start 2nd param,
        $BODY.= 'Content-Disposition: form-data; name="files"; filename="'.$filename.'"'. $eol; //Content data for post file
        $BODY.= 'Content-Type: application/octet-stream' . $eol. $eol; //content type
        $BODY.=$contents. $eol; // set the data of the file within boundary
        $BODY.= '--'.$BOUNDARY .'--' . $eol. $eol; // we close the boundary header.



        $ch = curl_init(); //init curl
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('X-Version: 1','Accept: multipart/form-data',
                                                 "Content-Type: multipart/form-data; boundary=".$BOUNDARY) //setting our mime type for make it work on $_FILE variable
                    );
        curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1); // call return content
        curl_setopt ($ch, CURLOPT_FOLLOWLOCATION, 1);// navigate the endpoint
        curl_setopt($ch, CURLOPT_POST, true); //set as post
        curl_setopt($ch, CURLOPT_INFILE, $handle); // set the file handle
        curl_setopt($ch, CURLOPT_INFILESIZE, $fsize); // set the file size
        curl_setopt($ch, CURLOPT_POSTFIELDS, $BODY); // set our $BODY 
        curl_setopt($ch,CURLOPT_TIMEOUT,30); //set time out
        curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,false);
        curl_setopt($ch,CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);

        $json = json_encode($request);
        print "Request:- ";
        print Dump($request);
        if (json_last_error()) {
            return array('code' => 400, 'info' => array('success' => 0, 'error' => "Failure encoding JSON request: " . json_last_error()));
        }

        // Must be sure to be trimmed before applying hmac.
        $json = preg_replace("/^\s+|\s+$/", "", $json);
        $hmac = hash('sha256', $json); // has the json request
        // Request signature
        $to_sign = $auth . "\n" . strtoupper($method) . "\n" . $path . "\n" . "" . "\n" . $hmac . "\n";
        
        $sig = hash_hmac('sha256', $to_sign, $p['secret']); // sign the request
        curl_setopt($ch, CURLOPT_COOKIE, "signature=" . $auth . ":" . $sig);  // set the cookie with signature
    
        curl_setopt($ch, CURLOPT_URL, "https://" . $p['host'] . $path .  ""); // set the host path
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST"); //set the path

        unset($code);
        $debug = $p['debug'];
	if ($debug) {
		try{
			
			
			$response = curl_exec($ch);
			$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			curl_close ($ch);
		}
		catch(Exception $e) {
			print "ERROR IN CURL REQUEST<br>";
		}
	}
	else{
		
		$response = curl_exec($ch);
	}
	
	unset($data);

	try {
		$data = (array)json_decode( $response );
	}
	catch(Exception $e) {
		if ($debug) {
			print "JSON Parse Exception: " . $e->getMessage() . "<br>";
		}
		$data = array( 'success' => 0, 'error' => "Non-JSON or unparsable data returned." );
	}

	return array('code' => $code, 'data' =>$data); }
	

		
	
////
//// Pretty printing of the API output 
////

//Function to print response
function Dump($items){
	print(json_encode($items) . "<br>");
}

////
//// Pretty printing of the Response output 
////

//Function to print response

function PrintResponse($resp){
    print "Response code:- " . $resp['code'];
    print "<br>Response: ";
    print Dump($resp['data']);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

////
//// API Access Methods
////


/*item ($code, $result) = Auth_Request($p);

$p:
   token:  Required. API Token
   secret: Required. API Secret
   host:   Required. API Hostname
   debug:  0|1, If true, emit debugging telemetry
   user:   Optional username
   pass:   Optional password

Returns the HTTP response code and a perl hash reference containing
the response data.

*/

//Function definition to get auth token
function Auth_Request($p) {
	$path = "/perl/api/v2/auth";

	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL,"https://" . $p['host'] . $path);
	curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");

	$date = time();
	unset($json);

	if ($p['user']) {
		$json = array(
			 'token'       => $p['token'],
			 'date'        => $date,
			 'user'        => $p['user'],
			 'pass'        => $p['pass'],
			 'signature'   => hash_hmac( 'sha256', $p['token'] . "\n" . $date . "\n" . $p['user'] . "\n" . $p['pass'] . "\n", $p['secret'] ),
		);
	}
	else {
		$json = array(
			 'token'       => $p['token'],
			 'date'        => $date,
			 'signature'   => hash_hmac('sha256', $p['token'] . "\n" . $date . "\n", $p['secret'] ),
		);
	}
	
	try {
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($json));
	}
	catch(Exception $e) {
		return array('code' => '400', 'info' => array( 'success' => 0, 'error' => "Failure encoding JSON request: " . $e->getMessage() ));
	}
	curl_setopt ($ch, CURLOPT_HTTPHEADER, Array("Content-Type: application/json",'Content-Length: ' . strlen(json_encode($json))));
	curl_setopt($ch, CURLOPT_HEADER, false);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);  
	return callAPI( $ch, $p['debug'] );
}


/*item ($code, $result) = Auth_Revoke($p);

$p:
   auth:   Required. API Authentication Code
   secret: Required. API Secret
   host:   Required. API Hostname
   debug:  0|1, If true, emit debugging telemetry

Returns the HTTP response code and a perl hash reference containing
the response data.

*/

//Function definition to revoke auth token
function Auth_Revoke($p) {
	$p['path']   = "/perl/api/v2/auth";
	$p['method'] = "DELETE";

	return API_Request($p);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*item ($code, $result) = API_Request($p);

$p:
   auth:   Required. API Authentication Code
   secret: Required. API Secret
   host:   Required. API Hostname
   debug:  0|1, If true, emit debugging telemetry
   method: HTTP Method to use.  E.g. GET, PUT, POST, ...
   path:   path of the request. E.g. "/perl/api/v2/user/username/settings/timezones"
   data:   Hash reference of data to send.

Data will be turned into a JSON for PUT/POST calls, and a query string
for all others.

Returns the HTTP response code and a perl hash reference containing
the response data.

*/

//Function definition to create api request
function API_Request($p) {
	$auth   = $p['auth'];
	$method = strtoupper((isset($p['method']) && $p['method'] !== '') ? $p['method'] : "GET");
	$path   = (isset($p['path']) && $p['path'] !== '') ? $p['path'] : "/perl/api/v2";
	$qs     = "";
	unset($json);
	unset($hmac);

	if(preg_match("/PUT|POST/",$method)) { 
		$json = json_encode($p['data']);
		if (json_last_error()) {
			return array('code' => 400, 'info' => array( 'success' => 0, 'error' => "Failure encoding JSON request: " . json_last_error() ));
        }
        Print 'Request: ';
        print  Dump($p['data']);
		// Must be sure to be trimmed before applying hmac.
		$json = preg_replace("/^\s+|\s+$/", "", $json);
		$hmac = hash('sha256', $json);
	}
	else {
		$list = array();
		if (array_key_exists("data",$p)){
			foreach ($p['data'] as $key => $value) {
			 array_push($list,URL_Encode($key) . "=" . URL_Encode($value));
			}
		}
		$qs = join("&",$list);
	}

	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL,"https://" . $p['host'] . $p['path'] . ($qs ? "?$qs" : ""));
	curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);

	// Send the POST/PUT body
	if (isset($json)) {
		curl_setopt ($ch, CURLOPT_HTTPHEADER, Array("Content-Type: application/json"));
		curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
	}

	// Request signature
	$to_sign = $auth . "\n" . strtoupper($method) . "\n" . $path . "\n" . $qs . "\n";
	if(isset($hmac)){
		$to_sign.= $hmac . "\n";
	}
	else{
		$to_sign.= NULL . "\n";
	}
	$sig = hash_hmac('sha256', $to_sign, $p['secret']);
	curl_setopt($ch, CURLOPT_COOKIE, "signature=" . $auth . ":" . $sig);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);  
	return callAPI( $ch, $p['debug'] );
}


//Function definition to make api request
function callAPI($ch,$debug) {
	curl_setopt($ch,CURLOPT_TIMEOUT,30);
	curl_setopt($ch, CURLOPT_USERAGENT, "LuxSci APIv2 PHP Client");
	curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,false);
    curl_setopt($ch,CURLOPT_SSL_VERIFYHOST, 2);

	
	unset($code);
	if ($debug) {
		try{
			
			
			$response = curl_exec($ch);
			$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			curl_close ($ch);
		}
		catch(Exception $e) {
			print "ERROR IN CURL REQUEST<br>";
		}
	}
	else{
		
		$response = curl_exec($ch);
	}
	
	unset($data);

	try {
		$data = (array)json_decode( $response );
	}
	catch(Exception $e) {
		if ($debug) {
			print "JSON Parse Exception: " . $e->getMessage() . "<br>";
		}
		$data = array( 'success' => 0, 'error' => "Non-JSON or unparsable data returned." );
	}

	return array('code' => $code, 'data' =>$data);
}


//////////////////////////////////////////////////////////
// URL-encode text
function URL_Encode($a) {
	 $b = preg_replace("([+?&=#<> %\"']|[^\x20-\x7E])", "%". to_hex(ord('$0')), $a);
   return $b;
}

# Convert number to 2-digit HEX
function to_hex($n) {
   $h = array( '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' );
   return $h[ intval($n/16) ] . $h[ intval($n % 16) ];
}

?>
